#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function


from libsignetsim.model.math.container.ListOfODEs import ListOfODEs
from libsignetsim.model.math.container.ListOfCFEs import ListOfCFEs
from libsignetsim.model.math.container.ListOfDAEs import ListOfDAEs
from libsignetsim.model.math.container.ListOfInitialConditions import ListOfInitialConditions
from libsignetsim.model.ListOfVariables import ListOfVariables
from libsignetsim.model.sbml.container.ListOfEvents import ListOfEvents
from libsignetsim.model.sbml.container.ListOfInitialAssignments import ListOfInitialAssignments
from libsignetsim.model.math.MathFormula import MathFormula


class MathSubmodel(object):
	""" Sbml model class """

	def __init__(self, parent_model=None):
		""" Constructor of model class """

		self.parentModel = parent_model
		self.sbmlLevel = self.parentModel.sbmlLevel
		self.sbmlVersion = self.parentModel.sbmlVersion

		self.clear()

	def clear(self):

		self.listOfODEs = ListOfODEs(self)
		self.listOfCFEs = ListOfCFEs(self)
		self.listOfDAEs = ListOfDAEs(self)
		self.listOfVariables = ListOfVariables(self)
		self.listOfEvents = ListOfEvents(self, self, math_only=True)
		self.listOfInitialAssignments = ListOfInitialAssignments(self, self, math_only=True)

		self.listOfInitialConditions = ListOfInitialConditions(self)
		self.listOfFunctionDefinitions = self.parentModel.listOfFunctionDefinitions
		self.nbOdes = None
		self.nbAssignments = None
		self.nbConstants = None
		self.nbAlgebraics = None

		self.variablesOdes = None
		self.variablesAssignment = None
		self.variablesConstant = None
		self.variablesAlgebraic = None

		self.__upToDate = False

	def isUpToDate(self):
		return self.__upToDate

	def setUpToDate(self, value):
		self.__upToDate = value

	def copyVariables(self):
		""" Copies the listOfVariables and the listOfInitialConditions """

		self.listOfVariables.copySubmodel(self.parentModel)

		for variable, value in list(self.parentModel.listOfInitialConditions.items()):
			t_value = MathFormula(self)
			t_value.setInternalMathFormula(value.getInternalMathFormula())
			self.listOfInitialConditions.update({variable: t_value})

	def copyEvents(self):
		self.listOfEvents.copySubmodel(self.parentModel.listOfEvents)

	def copyInitialAssignments(self):
		self.listOfInitialAssignments.copySubmodel(self.parentModel.listOfInitialAssignments)

	def prettyPrint(self):

		print("\n> Full system : ")
		print(self.listOfCFEs)
		print(self.listOfDAEs)
		print(self.listOfODEs)
		print(self.listOfInitialConditions)
		print("-----------------------------")

	def pprint(self):
		self.listOfCFEs.pprint()
		self.listOfDAEs.pprint()
		self.listOfODEs.pprint()
		self.listOfInitialConditions.pprint()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file is made to be called by the SBML Test Suite

"""
from __future__ import print_function


from libsignetsim.tests.sbmltestsuite.TestSuiteCase import TestSuiteCase
from libsignetsim.model.ModelException import LibSigNetSimException
from libsignetsim.simulation.SimulationException import SimulationException

from libsignetsim.settings.Settings import Settings

from sys import argv
from optparse import OptionParser

import traceback

class __main__(object):

	def __init__(self):

		if len(argv) > 1:
			self.readArgs(argv)


	def readArgs(self, argv):

		parser = OptionParser()

		parser.add_option("-v", "--verbose", dest="verbose", help="Verbose mode", nargs=0)

		parser.add_option("-k", "--keepfiles", dest="keep_files", help="Keep temporary files", nargs=0)

		parser.add_option("-c", "--test-conformity", dest="test_conformity", nargs=3,
						  help="Generate output for specified case. To use with Sbml Test Suite", metavar="CASE LEVEL VERSION")
		#
		# parser.add_option("-e", "--test-export", dest="test_export", nargs=3,
		# 				  help="Test sbml output for specified case. To use with Sbml Test Suite", metavar="CASE LEVEL VERSION")

		(options, _) = parser.parse_args()

		if options.verbose is not None:
			Settings.verbose = 1

		if options.keep_files is not None:
			Setting.simulationKeepFiles = True


		if options.test_conformity is not None:

			try:
				test = TestSuiteCase(
								int(options.test_conformity[0]),
								options.test_conformity[1],
								options.test_conformity[2]
				)

				test.runTestSuiteWraper()

			except LibSigNetSimException as e:
				print("Caught %s" % str(type(e)))
				print("> %s" % e.message)
				traceback.print_exc()

		# elif options.test_export is not None:
		#
		# 	try:
		# 		simulation = SbmlTestCaseSimulation(
		# 						int(options.test_export[0]),
		# 						options.test_export[1],
		# 						options.test_export[2],
		# 						test_export=True)
		# 		simulation.runTestSuiteWraper()
		#
		# 	except LibSigNetSimException as e:
		# 		print "Caught %s" % str(type(e))
		# 		print "> %s" % e.message
		# 		traceback.print_exc()
		#
		# 	except SimulationException as e:
		# 		print "Caught %s" % str(type(e))
		# 		print "> %s" % e.message
		# 		traceback.print_exc()



if __name__ == "__main__":
	__main__()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from libsignetsim.data.ExperimentalCondition import ExperimentalCondition
from libsignetsim.numl.NuMLDocument import NuMLDocument
from libsignetsim.figure.SigNetSimFigure import SigNetSimFigure
from matplotlib.pyplot import show


class Experiment(object):

	def __init__ (self, name=""):

		self.listOfConditions = {}
		self.currentId = 0
		self.name = name
		self.notes = ""

	def createCondition(self, name=""):
		condition = ExperimentalCondition(name)
		self.addCondition(condition)
		return condition

	def addCondition(self, condition):
		self.listOfConditions.update({self.currentId: condition})
		self.currentId += 1

	def readNuMLFromFile(self, filename):

		numl_doc = NuMLDocument()
		numl_doc.readNuMLFromFile(filename)
		result = numl_doc.listOfResultComponents[0]
		data = result.getDimensions()[0]
		self.name = data.getIndexValue()
		self.notes = data.getNotes()

		for data_condition in data.getContents():
			condition = self.createCondition(data_condition.getIndexValue())
			condition.readNuML(data_condition)

	def writeNuMLToFile(self, filename):

		numl_doc = NuMLDocument()

		time_term = numl_doc.listOfOntologyTerms.createOntologyTerm()
		time_term.defineAsTime()

		concentration_term = numl_doc.listOfOntologyTerms.createOntologyTerm()
		concentration_term.defineAsConcentration()

		result = numl_doc.listOfResultComponents.createResultComponent()

		self.writeNuMLDescription(result)
		experiment = result.createCompositeValue(result.getDimensionsDescriptions()[0], self.name)

		if self.notes is not None and len(self.notes) > 0:
			experiment.setNotes(self.notes)

		for condition in list(self.listOfConditions.values()):
			t_condition = experiment.createCompositeValue(experiment.getDescription().getContent(), condition.name)

			condition.writeNuML(t_condition)
		numl_doc.writeNuMLToFile(filename)

	def writeNuMLDescription(self, result_component):

		desc_experiment = result_component.createCompositeDescription("Experiment", "string")
		desc_condition = desc_experiment.createCompositeDescription("Condition", "string")
		desc_type = desc_condition.createCompositeDescription("Data type", "string")
		desc_time = desc_type.createCompositeDescription("Time", "double")
		desc_species = desc_time.createCompositeDescription("Species", "xpath")
		desc_values = desc_species.createTupleDescription()
		desc_values.createAtomicDescription("Concentration", "double")
		desc_values.createAtomicDescription("Standard deviation", "double")


	def getMaxTime(self):

		max_time = 0
		for condition in list(self.listOfConditions.values()):
			if condition.getMaxTime() > max_time:
				max_time = condition.getMaxTime()

		return max_time


	def getTimes(self):
		times = []
		for condition in list(self.listOfConditions.values()):
			times += condition.getTimes()
		return list(set(times))

	def getTreatedVariables(self):
		species = []
		for condition in list(self.listOfConditions.values()):
			species += condition.getTreatedVariables()

		if len(species) > 1:
			return list(set(species))
		else:
			return species

	def getVariables(self):
		species = []
		for condition in list(self.listOfConditions.values()):
			species += condition.getVariables()

		if len(species) > 1:
			return list(set(species))
		else:
			return species


	def plot(self, figure=None, suffix="", marker="-"):

		if figure is None:
			figure = SigNetSimFigure()

		plots = []
		for condition in list(self.listOfConditions.values()):
			plots.append(condition.plot(figure, suffix=suffix, marker=marker))

		show()

		return plots




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function

from libsignetsim.settings.Settings import Settings
from libsignetsim.cwriter.CWriterModels import CWriterModels
from libsignetsim.cwriter.CWriterData import CWriterData
from libsignetsim.simulation.SimulationException import SimulationException
from os.path import join, isfile
from os import mkdir
from shutil import copyfile
from time import time
from glob import glob


class CWriterSimulation(CWriterModels, CWriterData):

	def __init__ (self, list_of_models,
						time_min,
						list_samples,
						experiment,
						abs_tol,
						rel_tol):

		self.listOfModels = list_of_models
		self.timeMin = time_min
		self.listOfSamples = list_samples
		self.absTol = abs_tol
		self.relTol = rel_tol
		CWriterModels.__init__(self, self.listOfModels, self.timeMin, self.listSamples, self.absTol, self.relTol)

		if experiment is not None:
			CWriterData.__init__(self, [experiment], workingModel=list_of_models[0])
		else:
			CWriterData.__init__(self, experiment, workingModel=list_of_models[0])

		self.experiment = experiment

	def writeSimulationFiles(self):

		mkdir(join(self.getTempDirectory(), "src"))
		mkdir(join(self.getTempDirectory(), "lib"))

		# First the code
		mkdir(join(self.getTempDirectory(), "src/integrate"))
		copyfile(join(Settings.basePath, "lib/integrate/src/integrate.h"), join(self.getTempDirectory(), "src/integrate/integrate.h"))
		copyfile(join(Settings.basePath, "lib/integrate/src/models.h"), join(self.getTempDirectory(), "src/integrate/models.h"))
		copyfile(join(Settings.basePath, "lib/integrate/src/datas.h"), join(self.getTempDirectory(), "src/integrate/datas.h"))
		copyfile(join(Settings.basePath, "lib/integrate/src/realtype_math.h"), join(self.getTempDirectory(), "src/integrate/realtype_math.h"))
		copyfile(join(Settings.basePath, "lib/integrate/src/types.h"), join(self.getTempDirectory(), "src/integrate/types.h"))

		# Then the shared libraries
		if not isfile(join(Settings.basePath, "lib/integrate/integrate.so")):
			integrate_filename = glob(join(Settings.basePath, "lib", "integrate", "integrate*.so"))
			if len(integrate_filename) > 0:
				copyfile(integrate_filename[0], join(self.getTempDirectory(), "lib/integrate.so"))
			else:
				raise SimulationException("Could not find the numerical integration library. Please reinstall libSigNetSim")
		else:
			copyfile(join(Settings.basePath, "lib/integrate/integrate.so"), join(self.getTempDirectory(), "lib/integrate.so"))

		copyfile(join(Settings.basePath, "lib/templates/simulation/Makefile"), join(self.getTempDirectory(), "Makefile") )
		copyfile(join(Settings.basePath, "lib/templates/simulation/main.c"), join(self.getTempDirectory(), "src/main.c") )

		treated_variables_names = []
		if self.experiment is not None:
			treated_variables_names = self.experiment.getTreatedVariables()

		for modelInd, model in enumerate(self.listOfModels):
			treated_variables = []
			for name in treated_variables_names:
				if model.listOfVariables.containsName(name):
					treated_variables.append(model.listOfVariables.getByName(name).getSbmlId())
				elif model.listOfVariables.containsSbmlId(name):
					treated_variables.append(model.listOfVariables.getBySbmlId(name).getSbmlId())

			reduce = Settings.reduceByDefault

			if len(treated_variables) > 0:
				reduce = False

			model.build(vars_to_keep=treated_variables, reduce=reduce, tmin=self.timeMin[modelInd])

		start = time()
		self.writeModelFiles()
		self.writeDataFiles()

		if Settings.verboseTiming >= 1:
				print(">> Files written in %.2fs" % (time()-start))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.numl.NMBase import NMBase
from libsignetsim.numl.container.ListOfOntologyTerms import ListOfOntologyTerms
from libsignetsim.numl.container.ListOfResultComponents import ListOfResultComponents
from libsignetsim.numl.NuMLException import NuMLFileNotFound
from libsignetsim.settings.Settings import Settings

import libsbml, libnuml
from libnuml import readNUMLFromFile, writeNUML, NUMLDocument
from six.moves import reload_module
reload_module(libsbml)
from os.path import exists, dirname, basename

class NuMLDocument (NMBase):

	def __init__(self):
		NMBase.__init__(self, self)

		self.path = None
		self.filename = None

		self.listOfOntologyTerms = ListOfOntologyTerms(self)
		self.listOfResultComponents = ListOfResultComponents(self)


	def readNuML(self, numl_document, level=Settings.defaultNuMLLevel, version=Settings.defaultNuMLVersion):
		reload_module(libnuml)
		NMBase.readNuML(self, numl_document, level, version)
		self.listOfOntologyTerms.readNuML(numl_document.getOntologyTerms(), level, version)
		self.listOfResultComponents.readNuML(numl_document.getResultComponents(), level, version)
		reload_module(libsbml)
	def writeNuML(self, level=Settings.defaultNuMLLevel, version=Settings.defaultNuMLVersion):
		reload_module(libnuml)
		document = NUMLDocument(level, version)

		NMBase.writeNuML(self, document, level, version)

		self.listOfOntologyTerms.writeNuML(document, level, version)
		self.listOfResultComponents.writeNuML(document, level, version)
		reload_module(libsbml)
		return document

	def readNuMLFromFile(self, filename):

		if not exists(filename):
			raise NuMLFileNotFound("NuML file %s not found" % filename)

		document = readNUMLFromFile(filename)
		self.path = dirname(filename)
		self.filename = basename(filename)
		self.readNuML(document)

	def writeNuMLToFile(self, filename):

		self.path = dirname(filename)
		self.filename = basename(filename)

		document = self.writeNuML()
		writeNUML(document, filename)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	Testing the research for conservation laws, and the subsequent reduction

"""

from libsignetsim import Model, KineticLaw, MathFormula

from unittest import TestCase
from sympy import simplify


class TestRenameSbmlId(TestCase):
	""" Tests high level functions """


	def testRenameSbmlId(self):

		model = Model()

		c = model.listOfCompartments.new("c")

		s1 = model.listOfSpecies.new("s1")
		s2 = model.listOfSpecies.new("s2")
		s3 = model.listOfSpecies.new("s3")
		s4 = model.listOfSpecies.new("s4")
		s5 = model.listOfSpecies.new("s5")

		p1 = model.listOfParameters.new("p1")
		p2 = model.listOfParameters.new("p2")
		p3 = model.listOfParameters.new("p3")
		p4 = model.listOfParameters.new("p4")

		r1 = model.listOfReactions.new("reaction 1")
		r1.listOfReactants.add(s1)
		r1.listOfReactants.add(s2)
		r1.listOfProducts.add(s3)
		r1.setKineticLaw(KineticLaw.MASS_ACTION, reversible=True, parameters=[p1, p2])

		a1 = model.listOfRules.newAssignmentRule(s4, "p1*s3")
		rate1 = model.listOfRules.newRateRule(s5, "p4*s3")
		alg1 = model.listOfRules.newAlgebraicRule("p4*s3-p3")

		e1 = model.listOfEvents.new()
		e1.setTrigger("p2/s1 > p4/s3")
		e_ass = e1.addEventAssignment(s3, "p3*s3")


		model.renameSbmlId("s3", "s3_bis")

		fr1 = MathFormula(model)
		fr1.setPrettyPrintMathFormula("c * (p1 * (s1 / c) * (s2 / c) - p2 * s3_bis / c)", rawFormula=True)
		self.assertTrue(self.sympyEqual(
			r1.kineticLaw.getDefinition(rawFormula=True).getDeveloppedInternalMathFormula(),
			fr1.getDeveloppedInternalMathFormula()
		))

		fa1 = MathFormula(model)
		fa1.setPrettyPrintMathFormula("c * p1 * (s3_bis / c)", rawFormula=True)
		self.assertTrue(self.sympyEqual(
			a1.getDefinition(rawFormula=True).getDeveloppedInternalMathFormula(),
			fa1.getDeveloppedInternalMathFormula()
		))

		frate1 = MathFormula(model)
		frate1.setPrettyPrintMathFormula("c * p4 * (s3_bis / c)", rawFormula=True)
		self.assertTrue(self.sympyEqual(
			rate1.getDefinition(rawFormula=True).getDeveloppedInternalMathFormula(),
			frate1.getDeveloppedInternalMathFormula()
		))

		falg1 = MathFormula(model)
		falg1.setPrettyPrintMathFormula("-p3 + p4 * (s3_bis / c)", rawFormula=True)
		self.assertTrue(self.sympyEqual(
			alg1.getDefinition(rawFormula=True).getDeveloppedInternalMathFormula(),
			falg1.getDeveloppedInternalMathFormula()
		))

		fe_trigger = MathFormula(model)
		fe_trigger.setPrettyPrintMathFormula("p2/(s1/c) > p4/(s3_bis/c)", rawFormula=True)
		self.assertTrue(self.sympyEqual(
			e1.trigger.getDeveloppedInternalMathFormula(),
			fe_trigger.getDeveloppedInternalMathFormula()
		))

		fe_ass = MathFormula(model)
		fe_ass.setPrettyPrintMathFormula("p3*(s3_bis/c)", rawFormula=True)
		self.assertTrue(self.sympyEqual(
			e_ass.getDefinition().getDeveloppedInternalMathFormula(),
			fe_ass.getDeveloppedInternalMathFormula()
		))

	def sympyEqual(self, a, b):

		return simplify(a-b) == 0


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	Testing the research for conservation laws, and the subsequent reduction

"""

from libsignetsim import SbmlDocument

from unittest import TestCase
from os.path import join, dirname
from os import getcwd


class TestReduceRas(TestCase):
	""" Tests high level functions """


	def testReduceRas(self):

		testfiles_path = join(join(getcwd(), dirname(__file__)), "files")
		sbml_doc = SbmlDocument()
		sbml_doc.readSbmlFromFile(join(testfiles_path, "ras_das.xml"))

		sbml_model = sbml_doc.getModelInstance()
		sbml_model.build()
		sbml_model.pprint()

		sbml_model.stoichiometryMatrix.build()
		# sbml_model.stoichiometryMatrix.pprint()

		sbml_model.listOfConservationLaws.build()
		sbml_model.listOfConservationLaws.pprint()


		sbml_model.asymetricModel.build()
		sbml_model.asymetricModel.pprint()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	Test of combine archive : create a new combine archive

"""

from libsignetsim import CombineArchive, Settings

from unittest import TestCase
from os.path import join, dirname, basename


class TestCreateArchive(TestCase):

	def testCreateArchive(self):

		archive = CombineArchive()
		sbml_file = archive.addFile(join(dirname(__file__), "files", "00001-sbml-l3v1.xml"))
		sbml_file.setMaster()

		sedml_file = archive.addFile(join(dirname(__file__), "files", "00001-sbml-l3v1-sedml.xml"))
		sedml_file.setMaster()

		numl_file = archive.addFile(join(dirname(__file__), "files", "experiment_0.xml"))
		numl_file.setMaster()

		archive.writeArchive(join(Settings.tempDirectory, "test_00001.omex"))

		create_archive = CombineArchive()
		create_archive.readArchive(join(Settings.tempDirectory, "test_00001.omex"))

		files = create_archive.getListOfFiles()
		for file in files:

			if file.isSbml():
				self.assertTrue(file.getFilename() == basename(create_archive.getMasterSbml()))
				self.assertTrue(file.getFilename() == basename(create_archive.getAllSbmls()[0]))

			elif file.isSedml():
				self.assertTrue(file.getFilename() == basename(create_archive.getMasterSedml()))
				self.assertTrue(file.getFilename() == basename(create_archive.getAllSedmls()[0]))

			elif file.isNuml():
				self.assertTrue(file.getFilename() == basename(create_archive.getMasterNuml()))
				self.assertTrue(file.getFilename() == basename(create_archive.getAllNumls()[0]))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from re import sub

class ListOfSbmlVariables(object):
	""" Parent class for all the ListOf containers in a sbml model """

	def __init__(self, model):

		self.__model = model


	def names(self):
		""" Return a set of names of the sbml objects """
		return [obj.getName() for obj in self]

	def containsName(self, name):
		""" Test if a name is in the list """

		for obj in self:
			if name == obj.getName():
				return True
		return False


	def getByName(self, name, pos=0):
		""" Find sbml objects by their name """
		return [obj for obj in self if obj.getName() == name][pos]


	def sbmlIds(self):
		""" Return a set of ids of the sbml objects """
		return [obj.getSbmlId() for obj in self]


	def containsSbmlId(self, sbml_id):
		for var in self:
			# We have a risk of conflict with local parameters, who have a special scope. So we just ignore them here
			if var.getSbmlId() == sbml_id and not (var.isParameter() and var.isLocalParameter()):
				return True
		return False

	def getBySbmlId(self, sbml_id):
		for var in self:
			# We have a risk of conflict with local parameters, who have a special scope. So we just ignore them here
			if var.getSbmlId() == sbml_id and not (var.isParameter() and var.isLocalParameter()):
				return var

	def newSbmlId(self, variable, string=None):

		# If the string is empty, we choose the type of object as a root
		t_string = None
		if string == None:
			if variable.isSpecies():
				t_string = "species_%d" % variable.objId

			elif variable.isCompartment():
				t_string = "compartment_%d" % variable.objId

			elif variable.isReaction():
				t_string = "reaction_%d" % variable.objId

			elif variable.isEvent():
				t_string = "event_%d" % variable.objId

			elif variable.isParameter():
				if variable.isLocalParameter():
					t_string = "_local_%d_parameter_%d" % (variable.reaction.objId, variable.objId)
				else:
					t_string = "parameter_%d" % variable.objId

		# Otherwise we force the restriction on the string
		else:
			t_string = sub('[ ]+','_', string)
			t_string = sub('[^A-Za-z0-9_]+', '', t_string)

		# We check if the sbmlId already exist, and if so we add numbers !
		# THIS SUCKS
		if t_string in self.sbmlIds():
			i = 1
			while ("%s_%d" % (t_string, i)) in self.sbmlIds():
				i += 1
			t_string = "%s_%d" % (t_string, i)

		return t_string



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	Testing the reading/writing of miriam annotations of the model

"""

from libsignetsim import SbmlDocument, Settings
from libsignetsim.uris.URI import URI

from unittest import TestCase
from os.path import join, dirname, isdir
from os import mkdir, getcwd


class TestAnnotation(TestCase):
	""" Tests high level functions """


	def testReadWriteModel(self):

		testfiles_path = join(join(getcwd(), dirname(__file__)), "files")
		sbml_doc = SbmlDocument()
		sbml_doc.readSbmlFromFile(join(testfiles_path, "BIOMD0000000001.xml"))
		sbml_doc.writeSbmlToFile(join(Settings.tempDirectory, "BIOMD0000000001.xml"))
		sbml_doc.readSbmlFromFile(join(testfiles_path, "BIOMD0000000001.xml"))

		sbml_model = sbml_doc.getModelInstance()

		self.assertEqual(sbml_model.modelHistory.getDateCreated(), "2005-02-02T14:56:11Z")
		creator = sbml_model.modelHistory.getListOfCreators()[0]
		self.assertEqual(creator.getEmail(), "lenov@ebi.ac.uk")
		self.assertEqual(creator.getGivenName(), "Nicolas")
		self.assertEqual(creator.getFamilyName(), "Le Novère")


		taxon_uri = URI()
		taxon_uri.setTaxonomy('7787')
		self.assertEqual(sbml_model.getAnnotation().getHasTaxon()[0], taxon_uri)
		self.assertEqual(sbml_model.getAnnotation().getHasTaxon()[0].getName(), "Tetronarce californica")

		self.assertEqual(sbml_model.getAnnotation().getHasProperty(), [])
		self.assertEqual(sbml_model.getAnnotation().getHasPart(), [])
		self.assertEqual(sbml_model.getAnnotation().getHasVersion(), [])
		self.assertEqual(sbml_model.getAnnotation().getIs(), [])
		self.assertEqual(sbml_model.getAnnotation().getIsDescribedBy(), [])
		self.assertEqual(sbml_model.getAnnotation().getIsEncodedBy(), [])
		self.assertEqual(sbml_model.getAnnotation().getIsHomologTo(), [])
		self.assertEqual(sbml_model.getAnnotation().getIsPartOf(), [])
		self.assertEqual(sbml_model.getAnnotation().getIsPropertyOf(), [])

		go_process1_uri = URI()
		go_process1_uri.setGO('GO:0007274')
		go_process2_uri = URI()
		go_process2_uri.setGO('GO:0007166')
		go_process3_uri = URI()
		go_process3_uri.setGO('GO:0019226')
		self.assertEqual(sbml_model.getAnnotation().getIsVersionOf()[0], go_process1_uri)
		self.assertEqual(sbml_model.getAnnotation().getIsVersionOf()[1], go_process2_uri)
		self.assertEqual(sbml_model.getAnnotation().getIsVersionOf()[2], go_process3_uri)

		self.assertEqual(sbml_model.getAnnotation().getOccursIn(), [])
		self.assertEqual(sbml_model.getAnnotation().getUnknown(), [])
		self.assertEqual(sbml_model.getAnnotation().getModelHasInstance(), [])

		biomodels_ref1_uri = URI()
		biomodels_ref1_uri.setBiomodels('MODEL6613849442')
		biomodels_ref2_uri = URI()
		biomodels_ref2_uri.setBiomodels('BIOMD0000000001')

		self.assertEqual(sbml_model.getAnnotation().getModelIs()[0], biomodels_ref1_uri)
		self.assertEqual(sbml_model.getAnnotation().getModelIs()[1], biomodels_ref2_uri)

		self.assertEqual(sbml_model.getAnnotation().getModelIsDerivedFrom(), [])

		publication_uri = URI()
		publication_uri.setPubmed('8983160')
		self.assertEqual(sbml_model.getAnnotation().getModelIsDescribedBy()[0], publication_uri)
		self.assertEqual(
			sbml_model.getAnnotation().getModelIsDescribedBy()[0].getName(),
			("Edelstein SJ(1), Schaad O, Henry E, Bertrand D, Changeux JP., "
				+ "A kinetic mechanism for nicotinic acetylcholine receptors based on multiple allosteric transitions., "
				+ "1. Biol Cybern. 1996 Nov;75(5):361-79.")
		)
		self.assertEqual(sbml_model.getAnnotation().getModelIsInstanceOf(), [])
		self.assertEqual(sbml_model.getAnnotation().getModelUnknown(), [])

		descriptions = [
			u'physical compartment', u'protein complex', u'protein complex', u'protein complex',
			u'multimer of macromolecules', u'protein complex', u'multimer of macromolecules', u'protein complex',
			u'multimer of macromolecules', u'protein complex', u'protein complex', u'multimer of macromolecules',
			u'protein complex', u'non-covalent binding', u'non-covalent binding', u'conformational transition',
			u'non-covalent binding', u'non-covalent binding', u'conformational transition',
			u'conformational transition', u'non-covalent binding', u'non-covalent binding',
			u'conformational transition', u'conformational transition', u'conformational transition',
			u'non-covalent binding', u'non-covalent binding', u'physical compartment', u'protein complex',
			u'protein complex', u'protein complex', u'multimer of macromolecules', u'protein complex',
			u'multimer of macromolecules', u'protein complex', u'multimer of macromolecules', u'protein complex',
			u'protein complex', u'multimer of macromolecules', u'protein complex', u'non-covalent binding',
			u'non-covalent binding', u'conformational transition', u'non-covalent binding', u'non-covalent binding',
			u'conformational transition', u'conformational transition', u'non-covalent binding',
			u'non-covalent binding', u'conformational transition', u'conformational transition',
			u'conformational transition', u'non-covalent binding', u'non-covalent binding'
		]

		resolved_descriptions = sbml_model.getListOfSBOTermsDescriptions()
		self.assertEqual(resolved_descriptions, descriptions)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	Combine archive main object. 

"""



from .ListOfFiles import ListOfFiles
from .CombineException import (
	FileNotFoundException, NotAZipFileException, NoMasterSedmlFoundException, NoMasterSbmlFoundException,
	NoMasterNumlFoundException, NoSedmlFoundException
)
from libsignetsim.sedml.SedmlDocument import SedmlDocument
from libsignetsim.settings.Settings import Settings

from zipfile import is_zipfile, ZipFile
from random import choice
from string import ascii_uppercase, ascii_lowercase, digits
from os.path import exists, join, isdir, basename
from os import mkdir
from shutil import copy


class CombineArchive(object):
	""" Combine archive main object. """


	OMEX = "Default combine archive"
	SEDX = "SED-ML archive"
	SBEX = "SBML archive"
	CMEX = "CellML archive"
	NEUX = "NeuroML archive"
	PHEX = "PharmaML archive"
	SBOX = "SBOL archive"

	ARCHIVE_EXTENSIONS = {
		"omex": OMEX,
		"sedx": SEDX,
		"sbex": SBEX,
		"cmex": CMEX,
		"neux": NEUX,
		"phex": PHEX,
		"sbox": SBOX
	}

	SEDML = "sed-ml"

	def __init__(self):

		self.__file = None
		self.__listOfFiles = ListOfFiles(self)
		self.__path = Settings.tempDirectory + ''.join(choice(ascii_uppercase + ascii_lowercase + digits) for _ in range(12))
		if not isdir(self.__path):
			mkdir(self.__path)
		self.__extension = None

	def readArchive(self, file):
		""" Reads a combine archive file """
		if not exists(file):
			raise FileNotFoundException("File %s not found" % file)

		if not is_zipfile(file):
			raise NotAZipFileException("File %s is not a zip file" % file)

		self.__file = ZipFile(file)
		self.__extension = file.split(".")[-1]
		self.__listOfFiles.readListOfFiles(self.__file)

		self.__extractArchive()
		# self.__file.close()
		#TODO : browse the manifest, check if the archive exists, then create add the file to the list of files ?

	def getListOfFiles(self):
		""" Returns the list of files inside the combine archive """
		return self.__listOfFiles

	def __extractArchive(self):
		self.__file.extractall(self.__path)

	def writeArchive(self, filename):
		""" Writes the combine archive as a file """
		manifest_file = open(join(self.__path, "manifest.xml"), "w")
		manifest_file.write(self.__listOfFiles.writeManifest())
		manifest_file.close()

		self.__file = ZipFile(filename, mode='w')
		self.__file.write(join(self.__path, "manifest.xml"), "manifest.xml")

		for file in self.__listOfFiles.writeListOfFiles():

			self.__file.write(join(self.__path, file), file)

		self.__file.close()

	def getMasterSedml(self):
		""" Returns the master SEDML filename """
		if self.__listOfFiles.getMasterSedml() is not None:
			return join(self.__path, self.__listOfFiles.getMasterSedml().getFilename())
		else:
			raise NoMasterSedmlFoundException("No master Sedml found")

	def getAllSedmls(self):
		""" Returns all the SEDML filenames """

		all_sedmls = self.__listOfFiles.getAllSedmls()
		return [join(self.__path, sedml.getFilename()) for sedml in all_sedmls]

	def runMasterSedml(self):
		""" Execute the master SEDML and returns the SedmlDocument object """
		filename = self.getMasterSedml()

		if exists(filename):
			return self.runSedml(filename)

	def runAllSedmls(self):
		""" Execute all the SEDML documents and returns the SedmlDocument objects """
		all_sedmls = self.__listOfFiles.getAllSedmls()
		sedml_docs = []

		if len(all_sedmls) > 0:

			for sedml_file in all_sedmls:
				filename = join(self.__path, sedml_file.getFilename())
				if exists(filename):
					sedml_docs.append(self.runSedml(filename))

			return sedml_docs

		else:
			raise NoSedmlFoundException()

	def getMasterSedmlDoc(self):
		""" Returns the master SEDML as a SedmlDocument object """
		filename = self.getMasterSedml()

		if exists(filename):
			return self.__getSedmlDoc(filename)

	def __getSedmlDoc(self, filename):

		sedml_doc = SedmlDocument()
		sedml_doc.readSedmlFromFile(filename)
		return sedml_doc

	def runSedml(self, filename):
		""" Run the Sedml document """
		sedml_doc = self.__getSedmlDoc(filename)
		sedml_doc.run()
		return sedml_doc

	def getAllSbmls(self):
		""" Returns all the SBML filenames """
		all_sbmls = self.__listOfFiles.getAllSbmls()
		return [join(self.__path, sbml.getFilename()) for sbml in all_sbmls]

	def getMasterSbml(self):
		""" Returns the master SBML filename """
		if self.__listOfFiles.getMasterSbml() is not None:
			return join(self.__path, self.__listOfFiles.getMasterSbml().getFilename())
		else:
			raise NoMasterSbmlFoundException("No master SBML found")

	def getAllNumls(self):
		""" Returns all the NuML filenames """
		all_numls = self.__listOfFiles.getAllNumls()
		return [join(self.__path, numl.getFilename()) for numl in all_numls]

	def getMasterNuml(self):
		""" Returns the master NuML filename """
		if self.__listOfFiles.getMasterNuml() is not None:
			return join(self.__path, self.__listOfFiles.getMasterNuml().getFilename())
		else:
			raise NoMasterNumlFoundException("No master NuML found")

	def isDefaultCombineArchive(self):
		return self.__extension in self.ARCHIVE_EXTENSIONS and self.ARCHIVE_EXTENSIONS[self.__extension] == self.OMEX

	def isSEDMLArchive(self):
		return self.__extension in self.ARCHIVE_EXTENSIONS and self.ARCHIVE_EXTENSIONS[self.__extension] == self.SEDX

	def isSBMLArchive(self):
		return self.__extension in self.ARCHIVE_EXTENSIONS and self.ARCHIVE_EXTENSIONS[self.__extension] == self.SBEX

	def isCellMLArchive(self):
		return self.__extension in self.ARCHIVE_EXTENSIONS and self.ARCHIVE_EXTENSIONS[self.__extension] == self.CMEX

	def isNeuroMLArchive(self):
		return self.__extension in self.ARCHIVE_EXTENSIONS and self.ARCHIVE_EXTENSIONS[self.__extension] == self.NEUX

	def isPharmaMLArchive(self):
		return self.__extension in self.ARCHIVE_EXTENSIONS and self.ARCHIVE_EXTENSIONS[self.__extension] == self.PHEX

	def isSBOLArchive(self):
		return self.__extension in self.ARCHIVE_EXTENSIONS and self.ARCHIVE_EXTENSIONS[self.__extension] == self.SBOX

	def addFile(self, filename):
		copy(filename, join(self.__path, basename(filename)))
		return self.__listOfFiles.addFile(filename)
